@@ -162,14 +162,11 @@ gboolean key_event(unsigned int code, gboolean pressed, gboolean spi_enabled)
 {
 	START_FUNC
 	gboolean ret=TRUE;
+	flo_debug(TRACE_HIDEBUG, _("Send event %s on key [%d]"), pressed?_("press"):_("release"), code);
 #ifdef ENABLE_XTST
 	if (spi_enabled)
-#ifdef AT_SPI
 #ifdef ENABLE_AT_SPI2
 		ret=atspi_generate_keyboard_event(code, NULL, pressed?ATSPI_KEY_PRESS:ATSPI_KEY_RELEASE, NULL);
-#else
-		ret=SPI_generateKeyboardEvent(code, NULL, pressed?SPI_KEY_PRESS:SPI_KEY_RELEASE);
-#endif
 	if (!ret) flo_warn(_("ATSPI doesn't work. Using Xtst instead."));
 #else
 		flo_fatal(_("Unreachable code"));
@@ -181,12 +178,8 @@ gboolean key_event(unsigned int code, gboolean pressed, gboolean spi_enabled)
 		ret=FALSE;
 	}
 #else
-#ifdef AT_SPI
 #ifdef ENABLE_AT_SPI2
 	ret=atspi_generate_keyboard_event(code NULL, pressed?ATSPI_KEY_PRESS:ATSPI_KEY_RELEASE);
-#else
-	ret=SPI_generateKeyboardEvent(code, NULL, pressed?SPI_KEY_PRESS:SPI_KEY_RELEASE);
-#endif
 	if (!ret) flo_warn(_("ATSPI doesn't work."));
 #else
 #error _("Neither at-spi nor Xtest is compiled. You should compile one.")
@@ -321,6 +314,13 @@ void key_press(struct key *key, struct status *status)
 	END_FUNC
 }
 
+gboolean key_terminate(gpointer userdata)
+{
+	struct status *status=(struct status *)userdata;
+	status_terminate(status);
+	return FALSE;
+}
+
 /* Send a key release event. */
 void key_release(struct key *key, struct status *status)
 {
@@ -341,7 +341,8 @@ void key_release(struct key *key, struct status *status)
 			case KEY_ACTION:
 				action=(struct key_action *)mod->data;
 				switch (action->type) {
-					case KEY_CLOSE: gtk_main_quit(); break;
+					case KEY_CLOSE: g_timeout_add(20, key_terminate, (gpointer)status);
+						break;
 					case KEY_REDUCE: view_hide(status->view); break;
 					case KEY_CONFIG: settings(); break;
 					case KEY_MOVE: status_set_moving(status, FALSE); break;
