@@ -23,6 +23,7 @@
 #ifdef ENABLE_RAMBLE
 #include "trace.h"
 #include "settings.h"
+#include "style.h"
 #include <math.h>
 
 /* Maximum number of points in the ramble path */
@@ -31,6 +32,7 @@
 /* Invalidate the region modified by the path */
 void ramble_update_region(GList *path, GdkWindow *window)
 {
+	START_FUNC
 	GdkRectangle rect;
 	struct ramble_point *p1, *p2;
 
@@ -57,11 +59,13 @@ void ramble_update_region(GList *path, GdkWindow *window)
 			gdk_window_invalidate_rect(window, &rect, TRUE);
 		}
 	}
+	END_FUNC
 }
 
 /* Detect gesture based on distance */
 void ramble_distance(struct ramble *ramble)
 {
+	START_FUNC
 	GList *list;
 	gint dw, dh;
 	gdouble d=0.0, w, h, threshold;
@@ -85,18 +89,20 @@ void ramble_distance(struct ramble *ramble)
 			list=NULL;
 		}
 	}
-	if (pt2->ev) threshold=settings_double_get("behaviour/ramble_threshold2");
-	else threshold=settings_double_get("behaviour/ramble_threshold1");
-	threshold*=(settings_double_get("window/scalex")+settings_double_get("window/scaley"))/2.0;
+	if (pt2->ev) threshold=settings_get_double(SETTINGS_RAMBLE_THRESHOLD2);
+	else threshold=settings_get_double(SETTINGS_RAMBLE_THRESHOLD1);
+	threshold*=(settings_get_double(SETTINGS_SCALEX)+settings_get_double(SETTINGS_SCALEY))/2.0;
 	if (d>=threshold) {
 		pt->ev=TRUE;
 		list=NULL;
 	}
+	END_FUNC
 }
 
 /* Detect gesture based on time */
 void ramble_time(struct ramble *ramble)
 {
+	START_FUNC
 	struct ramble_point *pt;
 	pt=(struct ramble_point *)ramble->end->data;
 	if (pt->k) {
@@ -109,7 +115,7 @@ void ramble_time(struct ramble *ramble)
 			else
 				ramble->timer=g_timer_new();
 		} else if (g_timer_elapsed(ramble->timer, NULL)>=
-			(settings_double_get("behaviour/ramble_timer")/1000.0)) {
+			(settings_get_double(SETTINGS_RAMBLE_TIMER)/1000.0)) {
 			pt->ev=TRUE;
 			g_timer_start(ramble->timer);
 			g_timer_stop(ramble->timer);
@@ -118,25 +124,32 @@ void ramble_time(struct ramble *ramble)
 		g_timer_destroy(ramble->timer);
 		ramble->timer=NULL;
 	}
+	END_FUNC
 }
 
 /* Reset timer */
 void ramble_time_reset(struct ramble *ramble)
 {
+	START_FUNC
 	g_timer_start(ramble->timer);
+	END_FUNC
 }
 
 /* Add a point to the path and update the window.
  * returns TRUE if an event is detected. */
 gboolean ramble_add(struct ramble *ramble, GdkWindow *window, gint x, gint y, struct key *k)
 {
+	START_FUNC
 	gchar *val;
 	struct ramble_point *pt=g_malloc(sizeof(struct ramble_point));
 
 	/* Add the point to the path */
 	pt->p.x=x; pt->p.y=y; pt->k=k; pt->ev=FALSE;
 	ramble->end=g_list_append(ramble->end, (gpointer)pt);
-	if (!ramble->path) ramble->path=ramble->end;
+	if (!ramble->path) {
+		ramble->path=ramble->end;
+		pt->ev=TRUE;
+	}
 	if (ramble->end->next) ramble->end=ramble->end->next;
 	ramble->n++;
 	ramble_update_region(ramble->end, window);
@@ -149,7 +162,7 @@ gboolean ramble_add(struct ramble *ramble, GdkWindow *window, gint x, gint y, st
 	if (!k) return FALSE;
 
 	/* Gesture detection */
-	val=settings_get_string("behaviour/ramble_algo");
+	val=settings_get_string(SETTINGS_RAMBLE_ALGO);
 	if (!strcmp("distance", val))
 		ramble_distance(ramble);
 	else if (!strcmp("time", val))
@@ -161,27 +174,37 @@ gboolean ramble_add(struct ramble *ramble, GdkWindow *window, gint x, gint y, st
 	if (val) g_free(val);
 
 	ramble->started=TRUE;
+	END_FUNC
 	return pt->ev;
 }
 
-/* Start rambling. Note: when ramble_button is FALSE, ramble is always started. */
-void ramble_start(struct ramble *ramble)
+/* Start rambling. Note: when ramble_button is FALSE, ramble is always started.
+ * returns TRUE if an event is detected. */
+gboolean ramble_start(struct ramble *ramble, GdkWindow *window, gint x, gint y, struct key *k)
 {
+	START_FUNC
 	ramble->started=TRUE;
+	END_FUNC
+	return ramble_add(ramble, window, x, y, k);
 }
 
 /* Return TRUE if rambling is started */
 gboolean ramble_started(struct ramble *ramble)
 {
-	return (!settings_get_bool("behaviour/ramble_button")) || ramble->started;
+	START_FUNC
+	END_FUNC
+	return (!settings_get_bool(SETTINGS_RAMBLE_BUTTON)) || ramble->started;
 }
 
-/* Reset ramble path */
-void ramble_reset(struct ramble *ramble, GdkWindow *window)
+/* Reset ramble path
+ * returns TRUE if an event is detected. */
+gboolean ramble_reset(struct ramble *ramble, GdkWindow *window, struct key *k)
 {
+	START_FUNC
 	GdkRectangle *rect=NULL;
 	GList *list=ramble->path;
 	struct ramble_point *pt;
+	struct key *last=NULL;
 	while (list) {
 		pt=(struct ramble_point *)list->data;
 		if (!rect) {
@@ -196,6 +219,7 @@ void ramble_reset(struct ramble *ramble, GdkWindow *window)
 			if (pt->p.y<rect->y) { rect->height+=(rect->y-pt->p.y); rect->y=pt->p.y; }
 			else rect->height+=(pt->p.y-rect->y);
 		}
+		if (pt->ev) last=pt->k;
 		g_free(list->data);
 		list=list->next;
 	}
@@ -210,11 +234,14 @@ void ramble_reset(struct ramble *ramble, GdkWindow *window)
 	ramble->end=NULL;
 	ramble->n=0;
 	ramble->started=FALSE;
+	END_FUNC
+	return (last!=k);
 }
 
 /* Draw the ramble path to the cairo context */
 void ramble_draw(struct ramble *ramble, cairo_t *ctx)
 {
+	START_FUNC
 	GList *list=ramble->end;
 	struct ramble_point *p;
 	if (list) {
@@ -226,38 +253,44 @@ void ramble_draw(struct ramble *ramble, cairo_t *ctx)
 			cairo_line_to(ctx, p->p.x, p->p.y);
 			list=list->prev;
 		}
-		cairo_set_operator(ctx, CAIRO_OPERATOR_SOURCE);
+		cairo_set_operator(ctx, CAIRO_OPERATOR_OVER);
 		cairo_set_line_cap (ctx, CAIRO_LINE_CAP_ROUND);
 		cairo_set_line_join(ctx, CAIRO_LINE_JOIN_ROUND);
-		cairo_set_source_rgba (ctx, 1, 0, 0, 1);
+		style_cairo_set_color(ctx, STYLE_RAMBLE_COLOR);
 		cairo_set_line_width(ctx, 5);
 		cairo_stroke(ctx);
 	}
+	END_FUNC
 }
 
 /* called when the input method changes */
-void ramble_input_method_check(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void ramble_input_method_check(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct ramble *ramble=(struct ramble *)user_data;
-	gchar *val=settings_get_string("behaviour/input_method");
+	gchar *val=settings_get_string(SETTINGS_INPUT_METHOD);
 	if (strcmp("ramble", val))
-		ramble_reset(ramble, NULL);
+		ramble_reset(ramble, NULL, NULL);
 	if (val) g_free(val);
+	END_FUNC
 }
 
 /* Create a ramble structure */
 struct ramble *ramble_new()
 {
+	START_FUNC
 	struct ramble *ramble=g_malloc(sizeof(struct ramble));
 	if (!ramble) flo_fatal(_("Unable to allocate memory for ramble"));
 	memset(ramble, 0, sizeof(struct ramble));
-	settings_changecb_register("behaviour/input_method", ramble_input_method_check, ramble);
+	settings_changecb_register(SETTINGS_INPUT_METHOD, ramble_input_method_check, ramble);
+	END_FUNC
 	return ramble;
 }
 
 /* Destroy a ramble structure */
 void ramble_free(struct ramble *ramble)
 {
+	START_FUNC
 	GList *list=ramble->path;
 	while (list) {
 		g_free(list->data);
@@ -265,6 +298,7 @@ void ramble_free(struct ramble *ramble)
 	}
 	g_list_free(ramble->path);
 	g_free(ramble);
+	END_FUNC
 }
 
 #endif
