@@ -22,20 +22,24 @@
 #include <gio/gio.h>
 #include "view.h"
 
+typedef void (*service_cb)(void *);
+
 /* Service object */
 struct service {
 	GDBusConnection *connection;
 	guint owner_id;
 	GDBusNodeInfo *introspection_data;
 	struct view *view;
-	GCallback quit; /* Callback called to quit the applications (when the terminate method is called) */
+	service_cb quit; /* Callback called to quit the applications (when the terminate method is called) */
+	service_cb ready; /* Callback called when the service is ready. */
+	gpointer user_data;
 };
 
 /* Send the terminate signal */
 void service_terminate(struct service *service);
 
 /* Create a service object */
-struct service *service_new(struct view *view, GCallback quit);
+struct service *service_new(struct view *view, service_cb quit, service_cb ready, gpointer user_data);
 /* Destroy a service object */
 void service_free(struct service *service);
 
