@@ -38,6 +38,13 @@
 #include "tools.h"
 #include "settings-window.h"
 
+#ifdef ENABLE_AT_SPI2
+#define AT_SPI
+#endif
+#ifdef ENABLE_AT_SPI
+#define AT_SPI
+#endif
+
 static struct settings_window *settings_window=NULL;
 void settings_window_extension(GtkToggleButton *button, gchar *name);
 
@@ -167,16 +174,15 @@ void settings_window_preview_build()
 	END_FUNC
 }
 
-/* converts a color from string ro gdk */
-GdkColor *settings_window_convert_color(gchar *strcolor)
+/* converts a color from string to gdk */
+GdkRGBA *settings_window_convert_color(gchar *strcolor)
 {
 	START_FUNC
-	static GdkColor ret;
-	sscanf(strcolor, "#%02x%02x%02x",
-		(unsigned int *)&ret.red,
-		(unsigned int *)&ret.green,
-		(unsigned int *)&ret.blue);
-	ret.red<<=8; ret.green<<=8; ret.blue<<=8;
+	static GdkRGBA ret;
+	unsigned int r, g, b;
+	sscanf(strcolor, "#%02x%02x%02x", &r, &g, &b);
+	ret.red=((gdouble)r)/256.; ret.green=((gdouble)g)/256.; ret.blue=((gdouble)b)/256.;
+	ret.alpha=1.0;
 	END_FUNC
 	return &ret;
 }
@@ -341,8 +347,8 @@ void settings_window_update()
 					break;
 				case SETTINGS_COLOR:
 					color=settings_get_string(searchidx);
-					gtk_color_button_set_color(
-						GTK_COLOR_BUTTON(gtk_builder_get_object(settings_window->gtkbuilder,
+					gtk_color_chooser_set_rgba(
+						GTK_COLOR_CHOOSER(gtk_builder_get_object(settings_window->gtkbuilder,
 							params[searchidx].builder_name)),
 						settings_window_convert_color(color));
 					if (color) g_free(color);
@@ -444,14 +450,15 @@ void settings_window_style_change (GtkIconView *iconview, gpointer user_data)
 }
 
 /* on color change */
-void settings_window_change_color(GtkColorButton *button)
+void settings_window_change_color(GtkColorChooser *button)
 {
 	START_FUNC
-	GdkColor color;
+	GdkRGBA color;
 	gchar strcolor[8];
 	enum settings_item item=settings_get_settings_name(GTK_WIDGET(button));
-	gtk_color_button_get_color(button, &color);
-	g_sprintf(strcolor, "#%02X%02X%02X", (color.red)>>8, (color.green)>>8, (color.blue)>>8);
+	gtk_color_chooser_get_rgba(button, &color);
+	g_sprintf(strcolor, "#%02X%02X%02X", (int)((color.red)*256.),
+		(int)((color.green)*256.), (int)((color.blue)*256.));
 	settings_set_string(item, strcolor);
 	/* update style preview */
 	if (item==SETTINGS_KEY) settings_window_preview_build();
@@ -603,7 +610,7 @@ void settings_window_close(GtkWidget *window, GtkWidget *button)
 	closed=TRUE;
 	if (settings_dirty()) {
 		if (GTK_RESPONSE_ACCEPT==tools_dialog(_("Confirm"), GTK_WINDOW(window),
-			GTK_STOCK_APPLY, GTK_STOCK_DISCARD, _("Discard changes?")))
+			_("Apply"), _("Cancel"), _("Discard changes?")))
 			settings_window_commit(NULL, NULL);
 		else settings_window_rollback(NULL, NULL);
 	}
@@ -660,6 +667,13 @@ void settings_window_new(gboolean exit)
 	settings_window_layouts_populate();
 	settings_window_input_method_populate();
 
+#ifndef AT_SPI
+	gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(settings_window->gtkbuilder,
+		"flo_auto_hide")), FALSE);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(gtk_builder_get_object(settings_window->gtkbuilder,
+		"flo_auto_hide")), _("AT-SPI has been disabled at compile time, this option is not available."));
+#endif
+
 	settings_window_update();
 	settings_window->notify_id=settings_register_all(
 		(settings_callback)settings_window_update);
