@@ -79,7 +79,7 @@ static const struct settings_param settings_defaults[] = {
 	{ SETTINGS_WINDOW, "flo_transparent", "transparent", SETTINGS_BOOL, { .vbool = TRUE } },
 	{ SETTINGS_WINDOW, "flo_task_bar", "task-bar", SETTINGS_BOOL, { .vbool = FALSE } },
 	{ SETTINGS_WINDOW, "flo_always_on_top", "always-on-top", SETTINGS_BOOL, { .vbool = TRUE } },
-	{ SETTINGS_WINDOW, SETTINGS_NONE, "keep-on-top", SETTINGS_BOOL, { .vbool = TRUE } },
+	{ SETTINGS_WINDOW, "flo_keep_on_top", "keep-on-top", SETTINGS_BOOL, { .vbool = TRUE } },
 	{ SETTINGS_BEHAVIOUR, SETTINGS_NONE, "startup-notification", SETTINGS_BOOL, { .vbool = FALSE } },
 	{ SETTINGS_COLORS, "flo_keys", "key", SETTINGS_COLOR, { .vstring = "#000000" } },
 	{ SETTINGS_COLORS, SETTINGS_NONE, "outline", SETTINGS_COLOR, { .vstring = "#808080" } },
@@ -150,7 +150,7 @@ struct settings_registration *settings_registration_get(enum settings_item key)
 GSettings *settings_new_object(gchar *file, const gchar *cat)
 {
 	GSettingsBackend *backend;
-	GSettings *ret;
+	GSettings *ret=NULL;
 	if (file) {
 		backend=g_keyfile_settings_backend_new(file, "/apps/florence/", "toplevel");
 		if (!backend) {
@@ -340,7 +340,7 @@ gboolean settings_dirty()
 GVariant *settings_value_get(enum settings_item item)
 {
 	START_FUNC
-	GVariant *ret;
+	GVariant *ret=NULL;
 	gchar *str=NULL;
 	gchar *name=settings_defaults[item].settings_name;
 
