@@ -49,6 +49,7 @@ void view_show (struct view *view)
 	gtk_widget_show(GTK_WIDGET(view->window));
 	/* Some winwow managers forget it */
 	gtk_window_set_keep_above(view->window, TRUE);
+	gtk_window_set_urgency_hint(view->window, TRUE);
 	/* reposition the window */
 	gtk_window_move(view->window, settings_get_int(SETTINGS_XPOS), settings_get_int(SETTINGS_YPOS));
 #ifdef AT_SPI
@@ -74,6 +75,7 @@ void view_on_destroy(gpointer user_data)
 	START_FUNC
 	struct view *view=(struct view *)user_data;
 	view->window=NULL;
+	END_FUNC
 }
 
 /* destroy the view */
@@ -195,8 +197,8 @@ void view_keyboards_set_pos(struct view *view, struct keyboard *over)
 	START_FUNC
 	GSList *list=view->keyboards;
 	struct keyboard *keyboard;
-	gdouble width, height, xoffset, yoffset;
-	gdouble x, y;
+	gdouble width=0.0, height=0.0, xoffset=0.0, yoffset=0.0;
+	gdouble x=0.0, y=0.0;
 
 	/* browse the keyboards */
 	while (list)
@@ -208,7 +210,7 @@ void view_keyboards_set_pos(struct view *view, struct keyboard *over)
 				case LAYOUT_VOID:
 					width=keyboard_get_width(keyboard);
 					height=keyboard_get_height(keyboard);
-					xoffset=yoffset=0;
+					xoffset=yoffset=0.0;
 					x=y=0.0;
 					if (over) keyboard_set_under(keyboard); else keyboard_set_over(keyboard);
 					break;
@@ -301,9 +303,9 @@ struct key *view_hit_get (struct view *view, gint x, gint y)
 {
 	START_FUNC
 	GSList *list=view->keyboards;
-	struct keyboard *keyboard;
+	struct keyboard *keyboard=NULL;
 	struct key *key;
-	gint kx, ky, kw, kh;
+	gint kx=0.0, ky=0.0, kw=0.0, kh=0.0;
 
 	/* find the hit keyboard */
 	while (list)
@@ -855,6 +857,7 @@ void view_update_layout(struct view *view, struct style *style, GSList *keyboard
 	START_FUNC
 	view->style=style;
 	view->keyboards=keyboards;
+	xkeyboard_register_events(view->status->xkeyboard, view_on_keys_changed, (gpointer)view);
 	view_update_extensions(NULL, NULL, (gpointer)view);
 	END_FUNC
 }
